#ifndef EQ_H
#define EQ_H

#include <math.h>
//#include <stdio.h>

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

#define PEAKING 0
#define HPF 1

class EQFilter {
  
private:
  float a0, a1, a2, b0, b1, b2; // filter coefficients
  float x_1, x_2, y_1, y_2;	// last two samples
  
public:
  EQFilter(int type, float bw, float dBgain, float fs, float f0) {
    float a, w0, cosw0, sinw0, alpha, q;
    a = powf(10.0f ,(dBgain / 40.0f));
    w0 = 2.0f * M_PI * f0 / fs;
    cosw0 = cos(w0);
    sinw0 = sin(w0);

    q = 1.0f / sqrtf(2.0f);
    alpha = sinw0 / (2.0f * q);

    // alpha = sinw0 * sinh(ln(2)/2 * bw * w0 / sinw0);
    switch (type) {
    case PEAKING:
      b0 =   1.0f + alpha * a;
      b1 =  -2.0f * cosw0;
      b2 =   1.0f - alpha * a;
      a0 =   1.0f + alpha / a;
      a1 =  -2.0f * cosw0;
      a2 =   1.0f - alpha / a;
      break;
    case HPF:
      b0 =  (1.0f + cosw0) / 2.0f;
      b1 = -(1.0f + cosw0);
      b2 =  (1.0f + cosw0) / 2.0f;
      a0 =   1.0f + alpha;
      a1 =  -2.0f * cosw0;
      a2 =   1.0f - alpha;
      break;
    default:
      break;
    }

    //    fprintf(stderr, "b0=%f,b1=%f,b2=%f,a0=%f,a1=%f,a2=%f\n", b0,b1,b2,a0,a1,a2);

    x_1 = x_2 = y_1 = y_2 = 0.0f;
  }
  void do_filter(float *x, float *y, int len);
};

#endif
