/*
 * an experimental digital SSB generator
 *
 * Copyright (C) 2000-2006 Takafumi INOUE (JI3GAB) ji3gab@arrl.net
 * 
 * center freq.: 12kHz(default)
 * sample rate: 48kHz
 *
 * 2000.5.31
 * Java version
 *
 * 2000.10.7
 *
 * 2004
 * C++ again
 *
 * 2005.11
 * now can change filter bw on demand
 * 
 * 2006.2
 * add BFO 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>

#include "Generator.h"

#define BUFFER_SIZE 256
#define DECIMATION_FACTOR 4


void
Generator::generate(float *input, float *output_i, float *output_q, int n_samples) {
  
  static int decimation_factor = DECIMATION_FACTOR;

  // I/Q data buffer for lower sampling rate (12kHz)
  static float iLow[BUFFER_SIZE];
  static float qLow[BUFFER_SIZE];

  // I/Q data buffer for higher sampling rate (48kHz)
  static float iHigh[BUFFER_SIZE];
  static float qHigh[BUFFER_SIZE];

  // bfo buffer
  static float ibfo[BUFFER_SIZE], qbfo[BUFFER_SIZE];

  // decimation (to 12kHz)
  decimator->decimate(input, n_samples, iLow);
  int n_samples_low = n_samples / decimation_factor;

  // split into I/Q channel (just copying)
  for (int i = 0; i < n_samples_low; i++)
    qLow[i] = iLow[i];
	
  // I/Q filter
  // I signal is passed through a BPF
  iFilter->do_filter(iLow, n_samples_low);
  // Q signal is passed through a BPF which has the
  // same amplitude response as the one for I signal
  // but also has 90 degree phase shifter built in.
  qFilter->do_filter(qLow, n_samples_low);
      
  //fprintf(stderr, "%d samples low \n", n_samples_low);
      
  // interpolation (to 48kHz)
  iInterpolator->interpolate(iLow, n_samples_low, iHigh);
  qInterpolator->interpolate(qLow, n_samples_low, qHigh);

  // translates up to 12kHz
  if (ifreq != 0) {
    bfo->get_sample_n(ibfo, qbfo, n_samples);
  }

  // gain and phase adjuestment
  /*
  for (int i = 0; i < n_samples; i++) {
    qHigh[i] += phase_adj * iHigh[i];
    iHigh[i] *= (1 + gain_adj);
  }
  */
  for (int i = 0; i < n_samples; i++) {
    if (mode == 2) {
      if (ifreq != 0) {
	output_i[i] = iHigh[i] * ibfo[i] - qHigh[i] * qbfo[i];
	output_q[i] = iHigh[i] * qbfo[i] + qHigh[i] * ibfo[i];
	/*
	output_i[i] = ibfo[i];
	output_q[i] = qbfo[i];
	*/
      } else {
	output_i[i] = iHigh[i];
	output_q[i] = qHigh[i];
      }
    } else if (mode == 1){
      if (ifreq != 0) {
	output_i[i] = iHigh[i] * ibfo[i] + qHigh[i] * qbfo[i];
	output_q[i] = iHigh[i] * qbfo[i] - qHigh[i] * ibfo[i];
      } else {
	output_i[i] = iHigh[i];
	output_q[i] = -qHigh[i];
      }
    }
  }
  // gain and phase adjuestment

  for (int i = 0; i < n_samples; i++) {
    output_q[i] += phase_adj * output_i[i];
    output_i[i] *= (1 + gain_adj);
  }

}

void
Generator::set_iqfilter(float sf, float f1, float f2, float trband, float attn) {
  if (iFilter != NULL) {
    delete iFilter;
  }
  if (qFilter != NULL) {
    delete qFilter;
  }
  iFilter = new FIRFilter(IFIL, sf, f1, f2, trband, attn);
  qFilter = new FIRFilter(QFIL, sf, f1, f2, trband, attn);
}

void
Generator::filter_init() {
  float *a = (float *)(malloc(sizeof(float) * 83));  
  /*
   * Kaiser Window FIR Filter
   * Passband: 0.0 - 3000.0 Hz
   * Order: 83
   * Transition band: 1000.0 Hz
   * Stopband attenuation: 80.0 dB
   */
  a[0] =	-1.7250879E-5f;
  a[1] =	-4.0276995E-5f;
  a[2] =	-5.6314686E-5f;
  a[3] =	-4.0164417E-5f;
  a[4] =	3.0053454E-5f;
  a[5] =	1.5370155E-4f;
  a[6] =	2.9180944E-4f;
  a[7] =	3.6717512E-4f;
  a[8] =	2.8903902E-4f;
  a[9] =	3.1934875E-11f;
  a[10] =	-4.716546E-4f;
  a[11] =	-9.818495E-4f;
  a[12] =	-0.001290066f;
  a[13] =	-0.0011395542f;
  a[14] =	-3.8172887E-4f;
  a[15] =	9.0173044E-4f;
  a[16] =	0.0023420234f;
  a[17] =	0.003344623f;
  a[18] =	0.003282209f;
  a[19] =	0.0017731993f;
  a[20] =	-0.0010558856f;
  a[21] =	-0.004450674f;
  a[22] =	-0.0071515352f;
  a[23] =	-0.007778209f;
  a[24] =	-0.0053855875f;
  a[25] =	-2.6561373E-10f;
  a[26] =	0.0070972904f;
  a[27] =	0.013526209f;
  a[28] =	0.016455514f;
  a[29] =	0.013607533f;
  a[30] =	0.0043148645f;
  a[31] =	-0.009761283f;
  a[32] =	-0.02458954f;
  a[33] =	-0.03455451f;
  a[34] =	-0.033946108f;
  a[35] =	-0.018758629f;
  a[36] =	0.011756961f;
  a[37] =	0.054329403f;
  a[38] =	0.10202855f;
  a[39] =	0.14574805f;
  a[40] =	0.17644218f;
  a[41] =	0.18748334f;
  a[42] =	0.17644218f;
  a[43] =	0.14574805f;
  a[44] =	0.10202855f;
  a[45] =	0.054329403f;
  a[46] =	0.011756961f;
  a[47] =	-0.018758629f;
  a[48] =	-0.033946108f;
  a[49] =	-0.03455451f;
  a[50] =	-0.02458954f;
  a[51] =	-0.009761283f;
  a[52] =	0.0043148645f;
  a[53] =	0.013607533f;
  a[54] =	0.016455514f;
  a[55] =	0.013526209f;
  a[56] =	0.0070972904f;
  a[57] =	-2.6561373E-10f;
  a[58] =	-0.0053855875f;
  a[59] =	-0.007778209f;
  a[60] =	-0.0071515352f;
  a[61] =	-0.004450674f;
  a[62] =	-0.0010558856f;
  a[63] =	0.0017731993f;
  a[64] =	0.003282209f;
  a[65] =	0.003344623f;
  a[66] =	0.0023420234f;
  a[67] =	9.0173044E-4f;
  a[68] =	-3.8172887E-4f;
  a[69] =	-0.0011395542f;
  a[70] =	-0.001290066f;
  a[71] =	-9.818495E-4f;
  a[72] =	-4.716546E-4f;
  a[73] =	3.1934875E-11f;
  a[74] =	2.8903902E-4f;
  a[75] =	3.6717512E-4f;
  a[76] =	2.9180944E-4f;
  a[77] =	1.5370155E-4f;
  a[78] =	3.0053454E-5f;
  a[79] =	-4.0164417E-5f;
  a[80] =	-5.6314686E-5f;
  a[81] =	-4.0276995E-5f;
  a[82] =	-1.7250879E-5f;

  iInterpolator = new Interpolator(a, 83, DECIMATION_FACTOR);
  qInterpolator = new Interpolator(a, 83, DECIMATION_FACTOR);
  decimator = new Decimator(a, 83, DECIMATION_FACTOR);
  
  iFilter = new FIRFilter(IFIL, 12000.0f, 200.0f, 3000.0f, 200.0f, 60.0f);
  qFilter = new FIRFilter(QFIL, 12000.0f, 200.0f, 3000.0f, 200.0f, 60.0f);
}

void
Generator::set_ifreq(float freq) {
  ifreq = freq;
  bfo->set_freq(freq);
}
